# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:/bld/qgis_1624171887036/work/python/plugins/db_manager/ui/DlgCreateTable.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DbManagerDlgCreateTable(object):
    def setupUi(self, DbManagerDlgCreateTable):
        DbManagerDlgCreateTable.setObjectName("DbManagerDlgCreateTable")
        DbManagerDlgCreateTable.resize(474, 562)
        self.gridLayout_2 = QtWidgets.QGridLayout(DbManagerDlgCreateTable)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridlayout = QtWidgets.QGridLayout()
        self.gridlayout.setObjectName("gridlayout")
        self.label_3 = QtWidgets.QLabel(DbManagerDlgCreateTable)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridlayout.addWidget(self.label_3, 0, 0, 1, 2)
        self.cboSchema = QtWidgets.QComboBox(DbManagerDlgCreateTable)
        self.cboSchema.setObjectName("cboSchema")
        self.gridlayout.addWidget(self.cboSchema, 0, 2, 1, 1)
        self.label = QtWidgets.QLabel(DbManagerDlgCreateTable)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.gridlayout.addWidget(self.label, 1, 0, 1, 2)
        self.editName = QtWidgets.QLineEdit(DbManagerDlgCreateTable)
        self.editName.setText("")
        self.editName.setObjectName("editName")
        self.gridlayout.addWidget(self.editName, 1, 2, 1, 1)
        self.gridLayout_2.addLayout(self.gridlayout, 0, 0, 1, 2)
        self.vboxlayout = QtWidgets.QVBoxLayout()
        self.vboxlayout.setObjectName("vboxlayout")
        self.btnAddField = QtWidgets.QPushButton(DbManagerDlgCreateTable)
        self.btnAddField.setObjectName("btnAddField")
        self.vboxlayout.addWidget(self.btnAddField)
        self.btnDeleteField = QtWidgets.QPushButton(DbManagerDlgCreateTable)
        self.btnDeleteField.setObjectName("btnDeleteField")
        self.vboxlayout.addWidget(self.btnDeleteField)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.vboxlayout.addItem(spacerItem)
        self.btnFieldUp = QtWidgets.QPushButton(DbManagerDlgCreateTable)
        self.btnFieldUp.setObjectName("btnFieldUp")
        self.vboxlayout.addWidget(self.btnFieldUp)
        self.btnFieldDown = QtWidgets.QPushButton(DbManagerDlgCreateTable)
        self.btnFieldDown.setObjectName("btnFieldDown")
        self.vboxlayout.addWidget(self.btnFieldDown)
        self.gridLayout_2.addLayout(self.vboxlayout, 1, 1, 1, 1)
        self.hboxlayout = QtWidgets.QHBoxLayout()
        self.hboxlayout.setSpacing(8)
        self.hboxlayout.setObjectName("hboxlayout")
        self.label_4 = QtWidgets.QLabel(DbManagerDlgCreateTable)
        self.label_4.setObjectName("label_4")
        self.hboxlayout.addWidget(self.label_4)
        self.cboPrimaryKey = QtWidgets.QComboBox(DbManagerDlgCreateTable)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cboPrimaryKey.sizePolicy().hasHeightForWidth())
        self.cboPrimaryKey.setSizePolicy(sizePolicy)
        self.cboPrimaryKey.setObjectName("cboPrimaryKey")
        self.hboxlayout.addWidget(self.cboPrimaryKey)
        self.gridLayout_2.addLayout(self.hboxlayout, 2, 0, 1, 2)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.chkGeomColumn = QtWidgets.QCheckBox(DbManagerDlgCreateTable)
        self.chkGeomColumn.setObjectName("chkGeomColumn")
        self.gridLayout.addWidget(self.chkGeomColumn, 0, 0, 1, 1)
        self.cboGeomType = QtWidgets.QComboBox(DbManagerDlgCreateTable)
        self.cboGeomType.setObjectName("cboGeomType")
        self.cboGeomType.addItem("")
        self.cboGeomType.setItemText(0, "POINT")
        self.cboGeomType.addItem("")
        self.cboGeomType.setItemText(1, "LINESTRING")
        self.cboGeomType.addItem("")
        self.cboGeomType.setItemText(2, "POLYGON")
        self.cboGeomType.addItem("")
        self.cboGeomType.setItemText(3, "MULTIPOINT")
        self.cboGeomType.addItem("")
        self.cboGeomType.setItemText(4, "MULTILINESTRING")
        self.cboGeomType.addItem("")
        self.cboGeomType.setItemText(5, "MULTIPOLYGON")
        self.cboGeomType.addItem("")
        self.cboGeomType.setItemText(6, "GEOMETRYCOLLECTION")
        self.gridLayout.addWidget(self.cboGeomType, 0, 1, 1, 2)
        self.label_2 = QtWidgets.QLabel(DbManagerDlgCreateTable)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.editGeomColumn = QtWidgets.QLineEdit(DbManagerDlgCreateTable)
        self.editGeomColumn.setText("geom")
        self.editGeomColumn.setObjectName("editGeomColumn")
        self.gridLayout.addWidget(self.editGeomColumn, 1, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(DbManagerDlgCreateTable)
        self.label_5.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 2, 0, 1, 1)
        self.spinGeomDim = QtWidgets.QSpinBox(DbManagerDlgCreateTable)
        self.spinGeomDim.setMinimum(2)
        self.spinGeomDim.setMaximum(4)
        self.spinGeomDim.setObjectName("spinGeomDim")
        self.gridLayout.addWidget(self.spinGeomDim, 2, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(50, 51, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 2, 2, 2, 1)
        self.label_6 = QtWidgets.QLabel(DbManagerDlgCreateTable)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 3, 0, 1, 1)
        self.editGeomSrid = QtWidgets.QLineEdit(DbManagerDlgCreateTable)
        self.editGeomSrid.setText("0")
        self.editGeomSrid.setObjectName("editGeomSrid")
        self.gridLayout.addWidget(self.editGeomSrid, 3, 1, 1, 1)
        self.chkSpatialIndex = QtWidgets.QCheckBox(DbManagerDlgCreateTable)
        self.chkSpatialIndex.setObjectName("chkSpatialIndex")
        self.gridLayout.addWidget(self.chkSpatialIndex, 4, 0, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 3, 0, 1, 2)
        self.buttonBox = QtWidgets.QDialogButtonBox(DbManagerDlgCreateTable)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout_2.addWidget(self.buttonBox, 4, 0, 1, 2)
        self.fields = QtWidgets.QTableView(DbManagerDlgCreateTable)
        self.fields.setEditTriggers(QtWidgets.QAbstractItemView.AllEditTriggers)
        self.fields.setObjectName("fields")
        self.fields.horizontalHeader().setStretchLastSection(True)
        self.gridLayout_2.addWidget(self.fields, 1, 0, 1, 1)

        self.retranslateUi(DbManagerDlgCreateTable)
        self.buttonBox.rejected.connect(DbManagerDlgCreateTable.reject)
        QtCore.QMetaObject.connectSlotsByName(DbManagerDlgCreateTable)
        DbManagerDlgCreateTable.setTabOrder(self.cboSchema, self.editName)
        DbManagerDlgCreateTable.setTabOrder(self.editName, self.fields)
        DbManagerDlgCreateTable.setTabOrder(self.fields, self.btnAddField)
        DbManagerDlgCreateTable.setTabOrder(self.btnAddField, self.btnDeleteField)
        DbManagerDlgCreateTable.setTabOrder(self.btnDeleteField, self.btnFieldUp)
        DbManagerDlgCreateTable.setTabOrder(self.btnFieldUp, self.btnFieldDown)
        DbManagerDlgCreateTable.setTabOrder(self.btnFieldDown, self.cboPrimaryKey)
        DbManagerDlgCreateTable.setTabOrder(self.cboPrimaryKey, self.chkGeomColumn)
        DbManagerDlgCreateTable.setTabOrder(self.chkGeomColumn, self.cboGeomType)
        DbManagerDlgCreateTable.setTabOrder(self.cboGeomType, self.editGeomColumn)
        DbManagerDlgCreateTable.setTabOrder(self.editGeomColumn, self.spinGeomDim)
        DbManagerDlgCreateTable.setTabOrder(self.spinGeomDim, self.editGeomSrid)
        DbManagerDlgCreateTable.setTabOrder(self.editGeomSrid, self.chkSpatialIndex)
        DbManagerDlgCreateTable.setTabOrder(self.chkSpatialIndex, self.buttonBox)

    def retranslateUi(self, DbManagerDlgCreateTable):
        _translate = QtCore.QCoreApplication.translate
        DbManagerDlgCreateTable.setWindowTitle(_translate("DbManagerDlgCreateTable", "Create Table"))
        self.label_3.setText(_translate("DbManagerDlgCreateTable", "Schema"))
        self.label.setText(_translate("DbManagerDlgCreateTable", "Name"))
        self.btnAddField.setText(_translate("DbManagerDlgCreateTable", "Add field"))
        self.btnDeleteField.setText(_translate("DbManagerDlgCreateTable", "Delete field"))
        self.btnFieldUp.setText(_translate("DbManagerDlgCreateTable", "Up"))
        self.btnFieldDown.setText(_translate("DbManagerDlgCreateTable", "Down"))
        self.label_4.setText(_translate("DbManagerDlgCreateTable", "Primary key"))
        self.chkGeomColumn.setText(_translate("DbManagerDlgCreateTable", "Create geometry column"))
        self.label_2.setText(_translate("DbManagerDlgCreateTable", "Name"))
        self.label_5.setText(_translate("DbManagerDlgCreateTable", "Dimensions"))
        self.label_6.setText(_translate("DbManagerDlgCreateTable", "SRID"))
        self.chkSpatialIndex.setText(_translate("DbManagerDlgCreateTable", "Create spatial index"))
